/***************************************************************************
                          dspushbutton.cpp  -  description
                             -------------------
    begin                : Don Okt 30 2003
    copyright            : (C) 2003 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dspushbutton.h"

#include <kapplication.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kmimetype.h>
#include <kmessagebox.h>
#include <kpopupmenu.h>
#include <kurl.h>
#include <krun.h>

#include <qdragobject.h>
#include <qevent.h>
#include <qmime.h>
#include <qtooltip.h>
#include <qevent.h>

#define MENUID_START 1984

#define MENUID_CLEAR 1983
#define MENUID_REMOVE 1982
#define MENUID_ITEM 1981
#define MENUID_OPEN 1980

DSPushButton::DSPushButton(QWidget *parent, const char *name )
    : DCOPObject( "dragstack" ),
      QPushButton(parent,name)
{
    updateIcon();
    setAcceptDrops( true );
    
    m_menu = new KPopupMenu( this );
    m_menuItems = new KPopupMenu( m_menu );
    
    m_menu->insertTitle( i18n("DragStacK") );
    m_menu->insertItem( i18n("&Clear Stack"), this, SLOT( clear() ), 0, MENUID_CLEAR );
    m_menu->insertItem( SmallIconSet( "edittrash" ), i18n("&Remove Item"), this, SLOT( remove() ), 0, MENUID_REMOVE );
    m_menu->insertSeparator();
    m_menu->insertItem( i18n("Select Item"), m_menuItems, MENUID_ITEM );
    m_menu->insertSeparator();
    m_menu->insertItem( SmallIconSet( "fileopen" ), i18n("&Open"), this, SLOT( run() ), 0, MENUID_OPEN );
    m_menu->insertSeparator();
    m_menu->insertItem( SmallIconSet( "dragstack" ), i18n("&About..."), this, SIGNAL( about() ) );

    connect( m_menuItems, SIGNAL( activated(int) ), this, SLOT( setTopItem(int) ) );
    connect( this, SIGNAL( clicked() ), this, SLOT( run() ) );
}

DSPushButton::~DSPushButton()
{
}

void DSPushButton::dragEnterEvent(QDragEnterEvent* event)
{
    event->accept( QUriDrag::canDecode(event) );
}

void DSPushButton::dropEvent(QDropEvent* event)
{
    t_StackItem item;
    
    if ( QUriDrag::decodeToUnicodeUris(event, item.list) ) 
    {
        item.eType = t_StackItem::URL;
        
        m_files.push( item );
        updateIcon();
    }
    else
    {
        item.eType = t_StackItem::MIME;
        
    }
}

void DSPushButton::mousePressEvent( QMouseEvent* e )
{
    m_presspos = e->pos();
    m_pressed = ( e->button() == Qt::LeftButton );
        
    QPushButton::mousePressEvent( e );
}

void DSPushButton::contextMenuEvent( QContextMenuEvent* e )
{
    QString t;
    bool enable = !m_files.isEmpty();
    
    m_menu->setItemEnabled( MENUID_CLEAR, enable );
    m_menu->setItemEnabled( MENUID_REMOVE, enable );
    m_menu->setItemEnabled( MENUID_ITEM, enable );
    m_menu->setItemEnabled( MENUID_OPEN, enable );

    m_menuItems->clear();
    if( !m_files.isEmpty() )
        for( int i = (signed int)m_files.count() - 1; i >= 0; i-- )
        {
            t_StackItem item = m_files[i];
            if( item.eType == t_StackItem::URL )
            {
                if( item.list.count() > 1 )
                    t = QString( i18n("&%1 %2 ..." ) ).arg( (signed int)m_files.count() - i ).arg( item.list.first() );
                else
                    t = QString( i18n("&%1 %2" ) ).arg( (signed int)m_files.count() - i ).arg( item.list.first() );
            }
                        
            m_menuItems->insertItem( t , i + MENUID_START );
        }
        
    m_menu->popup( e->globalPos() );
}

void DSPushButton::mouseMoveEvent( QMouseEvent* e )
{
    if ( !m_files.isEmpty() && m_pressed && 
         ( m_presspos - e->pos() ).manhattanLength() > KApplication::startDragDistance() ) 
    {
        t_StackItem item = m_files.pop();

        if( item.eType == t_StackItem::URL )
        {
            QUriDrag* drag = new QUriDrag( this );
            drag->setUnicodeUris( item.list );
            drag->drag();
        }
        
        m_pressed = false;
        updateIcon();
    }
    
    QPushButton::mouseMoveEvent( e );
}

void DSPushButton::mouseReleaseEvent( QMouseEvent* e )
{
    m_pressed = false;
    updateIcon();
    
    QPushButton::mouseReleaseEvent( e );
}

void DSPushButton::updateIcon()
{
    QString tt = QString::null;
    QPixmap pix;
    
    if( !m_files.isEmpty() ) 
    {
        t_StackItem item = m_files.last();
        pix = kapp->iconLoader()->loadIcon( KMimeType::iconForURL( KURL( item.list.first() ) ), KIcon::Panel );

        tt = i18n("<qt><b>Contents:</b><br><br>");
        for( unsigned int i = 0; i < item.list.count(); i++ )
            tt.append( "<nobr>" + item.list[i] + "</nobr><br>" );
            
        tt.append( "</qt>" );
    } 
    else
    {
        pix = kapp->iconLoader()->loadIcon( "empty", KIcon::Panel );
        tt = i18n("The stack is empty.");
    }
        
    setPixmap( pix );
    QToolTip::remove( this );
    QToolTip::add( this, tt );
}

void DSPushButton::clear()
{
    m_files.clear();
    updateIcon();
}

void DSPushButton::remove()
{
    m_files.remove( m_files.last() );
    updateIcon();
}

int DSPushButton::count() const
{
    return m_files.count();
}

void DSPushButton::push( KURL url )
{
    if( url.isValid() ) 
    {
        t_StackItem item;
        item.eType = t_StackItem::URL;
        item.list = QStringList( url.path() );
        
        m_files.push( item );
        updateIcon();
    }
}

KURL DSPushButton::pop()
{
    if( !m_files.isEmpty() )
    {
        t_StackItem item = m_files.pop();
        updateIcon();

        // Uh, bad code ... me might lose data
        return KURL( item.list.first() );
    }
    else 
        return NULL;
}

void DSPushButton::run()
{
    // TODO: start every file if there are more than one...
    if( !m_files.isEmpty() ) 
    {
        t_StackItem item = m_files.last();
        
        if( item.eType == t_StackItem::URL )
        {
            KRun* run = new KRun( item.list.first() );
            run->setAutoDelete( true );
        }
    }
}

void DSPushButton::setTopItem( int id )
{
    t_StackItem item = m_files[id-MENUID_START];
    m_files.remove( m_files.at( id-MENUID_START ) );
    m_files.push( item );
    updateIcon();
}
