/***************************************************************************
                          dspushbutton.h  -  description
                             -------------------
    begin                : Don Okt 30 2003
    copyright            : (C) 2003 by Dominik Seichter
    email                : domseichter@web.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DSPUSHBUTTON_H
#define DSPUSHBUTTON_H

#include <qwidget.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qvaluestack.h>
#include <qstring.h>

#include "dragiface.h"

typedef struct t_StackItem 
{
    enum EType { URL, MIME } eType;
    QStringList list;

    void operator=( const t_StackItem & s )
    {
        this->eType = s.eType;
        this->list = s.list;
    }
    
    bool operator==( const t_StackItem & s )
    {
        return (this->eType == s.eType && this->list == s.list);
    }
};

class KPopupMenu;
class QContexMenuEvent;
class QDragEnterEvent;
class QDropEvent;
class QMouseEvent;

/**
  *@author Dominik Seichter
  */

class DSPushButton : public QPushButton, public DragIface {
    Q_OBJECT
    public: 
        DSPushButton(QWidget *parent=0, const char *name=0);
        ~DSPushButton();

        void updateIcon();

    public slots:
        void clear();
        int count() const;
        void run();

        void push( KURL url );
        KURL pop();

    signals:
        void about();

    private slots:
        void remove();
        void setTopItem(int i);

    protected:
        void dragEnterEvent(QDragEnterEvent* event);
        void dropEvent(QDropEvent* event);
        void mouseMoveEvent( QMouseEvent* e );
        void mousePressEvent( QMouseEvent* e );
        void mouseReleaseEvent( QMouseEvent* e );
        void contextMenuEvent( QContextMenuEvent* e );
        
    private:
        KPopupMenu* m_menu;
        KPopupMenu* m_menuItems;
        
        QPoint m_presspos;
        bool m_pressed;
        QValueStack<t_StackItem> m_files;
};

#endif
